/** A mapping from K to V. Null keys are not allowed. */
interface MapI<K,V> {
//  /* A key-value pair in a mapping. */
//  static interface Entry<K,V> {
//    /** @return the key for this entry. */
//    K getKey();
//    /** @return the value for this entry. */
//    V getValue();
//    /** Sets the value for this entry to v. An optional method?
//      * @return the old value for this entry if it existed, null otherwise. 
//      */
//    V setValue(V v);
//  }
  /** create a new empty map of the same type. */
  MapI<K,V> newMap();
  /** @return true if the map is empty (contains no key-value pairs). */
  boolean isEmpty();
  /** Set this map to the empty map. */
  void clear();
  /** @return the value for key if there is an entry (key,value) in the map; null otherwise. */
  V get(K key);
  /** Puts the entry (key, value) in the map, replacing the old entry for key if one existed.
    * @return the old value for key or null if no old value existed in the map.
    */
  V put(K key, V value);
  /* Deletes the entry for key.
   * @return the value of the removed entry if such an entry existed, null otherwise. */
  V remove(K key);
  /* return true iff an entry for key exists in the map. */
  boolean containsKey(K key);
//  /* return true iff an entry for key exists in the map. */
//  boolean containsValue(K key);
  /** Returns the number of entries (keys) in this map. */
  int size();
//  /** Returns the set of keys in this map. */
//  SetI<K> keySet();
//  /** @return the entry set for this map.  */
//  SetI<Map.EntryI<K,V>> entrySet();
//  /** @return an enumeration of the values in this map. */
//  EnumI<V> values();
}

/* A map over a totally-ordered domain K. */
interface SortedMapI<K extends Comparable<K>, V> {
  K firstKey();
  MapEntry firstEntry();
  K lastKey();
  MapEntry lastEntry();
}

/** A mutable binary search tree of elements of type T. */ 
class TreeMap<K, V> implements SortedMapI { 
  /** Number of entries in the map. */
  private int size;
  /** Root of the binary search tree. */
  private Node root;
  
  /* Relying on default constructor. Sets root to null; size to 0. */
  
  /** create a new empty map of the same type. */
  MapI<K,V> newMap() { return new TreeMap<K,V>(); }
  
  /** @return true if the map is empty (contains no key-value pairs). */
  boolean isEmpty() { return root == null; }
  
  /** Set this map to the empty map. */
  void clear() { root = null; }
  
  /** Assumes the map is non-empty. As a result, never returns null.
    * @returns the node that contains key if key exists in map; otherwise
    *          the parent for the key node if key were to be inserted in map. */
  private Node find(K key) {
    Node current = root;
    Node parent = null;
    while (current != null) {
      int compareFlag = key.compareTo(current.key);
      if (compareFlag == 0) break;
      parent = current;
      if (compareFlat < 0) current = current.left;
      else current = current.right;
    }
    if (current == null) return parent;
    return current;
  }
  
  /** @return the value for key if there is an entry (key,value) in the map; null otherwise. */
  public V get(K key) {
    Node match = find(key);
    if (match.key.equals(key)) return match.value;
    return null;
  }
  
  /** Puts the entry (key, value) in the map, replacing the old entry for key if one existed.
    * @return the old value for key or null if no old value existed in the map.
    */
  public V put(K key, V value) {
    if (root == null) {
      root = new Node(key, value);
      size = 1;
      return null
    }
    
    Node nearMatch = find(key);
    int compareFlag = key.compareTo(nearMatch.key);
    if (compareFlag == 0) {  // exact match; key already exists in map
      V oldValue = match.value;
      match.value = value;
      // size is unchanged
      return oldValue;
    }
    else { // Key is not present, must add new node
      if (compareFlag < 0) match.left = new Node(key, value);
      else match.right = new Node(key, value);node
      size++;
    return null;
  }
      
//  /* Deletes the entry for key.
//   * @return the value of the removed entry if such an entry existed, null otherwise. */
//  public V remove(K key);
//    if (root == null) return null;
//    ...
//  }
  
  /* return true iff an entry for key exists in the map. */
    boolean containsKey(K key) {
      V value = get(key);
      return value != null;
    }
    
//  /* return true iff an entry for key exists in the map. */
//  boolean containsValue(K key);
    
  /** Returns the size of this map. */
  int size() { return size; }
//  /** Returns the set of keys in this map. */      
//  SetI<K> keySet();
//  /** @return the entry set for this map.  */
//  SetI<Map.EntryI<K,V>> entrySet();
//  /** @return an enumeration of the values in this map. */
//  EnumI<V> values();
}
