import java.util.ArrayList;

/** An ordered extension of ArrayList. Uses the natural ordering on E.
  * Class invariant: the elements of this ArrayList appear in ascending (non-descending) order.
  * Note: this may contain equal elements; a find operation searching for a duplicated element 
  * returns one of the possible matches. */
class OrderedArrayList<E extends Comparable<E>> extends ArrayList<E> {
  
  public static final int FAIL = -1;
  
  public void insert(E item) { 
    int i = size();  // i = 1 + index of last element
    add(item);  // expand the ArrayList by one; item used as dummy value
    assert size() > 0;  // ArrayList is now non-empty
    // Invariant: i is index of open slot; original elements are stored in [0:i-1] [i+1:size()-1]
    while (i > 0 && item.compareTo(get(i-1)) < 0) {
      set(i, get(i-1));   // move up elt from index i-1 to i
      i--;
    }
    set(i, item);
  }
  
  public int find(E item) { return findHelp(item, 0, size() - 1); }
  
    /** @return the index of a matching entry; FAIL (-1) if no match */
  private int findHelp(E item, int lo, int hi) {
    if (lo > hi) return FAIL;
    int mid = (lo + hi) / 2;  // When lo == hi, mid == lo == hi
    int compareFlag = item.compareTo(get(mid));
    if (compareFlag == 0) return mid;
    else if (compareFlag < 0) return findHelp(item, lo, mid-1);
    else return findHelp(item, mid + 1, hi);
  }
  
  public int loopFind(E item) { return loopFindHelp(item, 0, size() - 1); }
  
  private int loopFindHelp(E item, int lo, int hi) {
    while (lo <= hi) {
      int mid = (lo + hi) / 2;
      int compareFlag = item.compareTo(get(mid));
      if (compareFlag == 0) return mid;
      if (compareFlag < 0) hi = mid - 1;
      else lo = mid + 1;
    }
    return FAIL;
  }
}
      
                         
    
    