import junit.framework.TestCase;

/** Test case class for the List composite classes with map, insert, sort, and merge. */

public class ListTest extends TestCase {

  public final static List<Integer> E = new EmptyList<Integer>();
  public final static List<Integer> l1 = E.cons(17);                                      // '(17)
  public final static List<Integer> l2 = l1.cons(5);                                      // '(5 17)
  public final static List<Integer> l3 = E.cons(5);                                       // '(5)
  public final static List<Integer> l4 = l3.cons(5);                                      // '(5 5)
  public final static List<Integer> l5 = l3.cons(17);                                     // '(17 5)
  public final static List<Integer> l6 = l5.cons(-5).cons(100).cons(50);                  // '(50 100 -5 17 5)
  public final static List<Integer> l7 = E.cons(100).cons(50).cons(17).cons(5).cons(-5);  // '(-5 5 17 50 100)
  public final static List<Integer> l8 = E.cons(20).cons(0);                              // '(0 20)
  public final static List<Integer> l9 = E.cons(100).cons(50).cons(20).cons(17).cons(5).cons(0).cons(-5); // '(-5 0 5 17 20 50 100)
  
  /** Tests the methods in List<E>. */
  public void testBasic() {
      List<Integer> l10 = l2.cons(-5);
      assertEquals("l2.cons(-5)", "(-5 5 17)", l10.toString());
      assertEquals("l1.cons(5)", l2, l1.cons(5));
      assertEquals("l3.cons(17)", l5, l3.cons(17));
//      assertEquals("l5.concat(cons(100).cons(50)", l7, l10.concat(E.cons(100).cons(50)));
  }
  /** Tests the InsertVisitor operation. */
  public void testInsertVisitor() {
    assertEquals("empty insert test", l1, E.accept(new InsertVisitor<Integer>(17)));
    assertEquals("pre insert test", l2, l1.accept(new InsertVisitor<Integer>(5)));
    assertEquals("post insert test", l2, l3.accept(new InsertVisitor<Integer>(17)));
    assertEquals("equals insert test", l4, l3.accept(new InsertVisitor<Integer>(5)));
  }
  /** Tests the MergeVisitor operation. */
  public void testMergeVisitor() {
    assertEquals("empty both", E, E.accept(new MergeVisitor<Integer>(E)));
    assertEquals("empty this", l3, E.accept(new MergeVisitor<Integer>(l3)));
    assertEquals("empty other", l3, l3.accept(new MergeVisitor<Integer>(E)));
    assertEquals("singletons", l2, l3.accept(new MergeVisitor<Integer>(l1)));
    assertEquals("opposite singletons", l2, l1.accept(new MergeVisitor<Integer>(l3)));
    assertEquals("big merge", l9, l7.accept(new MergeVisitor<Integer>(l8)));
    assertEquals("opposite big merge", l9, l8.accept(new MergeVisitor<Integer>(l7)));
  } 
  
  /** Tests the map method. */
  public void testMap() {
    ILambda<Integer, Integer> square = 
      new ILambda<Integer, Integer>() { public Integer apply(Integer i) { return i*i; } };
    assertEquals("l8 squared", "(0 400)", l8.map(square).toString());
  }
}
  


