import java.util.ArrayList;

class BinSearch<E extends Comparable<E>> {
  
  public static final int FAIL = -1;
  
  ArrayList<E> table = new ArrayList<E>();
  
  void insert(E item) { 
    int i = table.size();  // 1 + index of last element of table
    table.add(item);  // expand the ArrayList by one; item used as dummy value
    assert table.size() > 0;
    // Invariant: i is index of open slot; table' == table[0:i-1] + table[i+1; table'.size() - 1]
    while (i > 0 && item.compareTo(table.get(i-1)) < 0) {
      // move up elt from index i-1 to i
      table.set(i, table.get(i-1));
      i--;
    }
    table.set(i, item);
  }
  
  /** Assumes that table is sorted in ascending order and contains no equal keys
    * @return the index of the entry in table that equals item if a match is found; otherwise FAIL (-1) */
  int find(E item, int lo, int hi) {
    if (lo > hi) return FAIL;
    int mid = (lo + hi) / 2;  // When lo = hi, mid = lo = hi
    int compareFlag = item.compareTo(table.get(mid));
    if (compareFlag == 0) return mid;
    else if (compareFlag < 0) return find(item, lo, mid-1);
    else return find(item, mid + 1, hi);
  }
  
  public String toString() { return table.toString(); }
  
  int loopFind(E item, int lo, int hi) {
    while (lo <= hi) {
      int mid = (lo + hi) / 2;
      int compareFlag = item.compareTo(table.get(mid));
      if (compareFlag == 0) return mid;
      if (compareFlag < 0) hi = mid - 1;
      else lo = mid + 1;
    }
    return FAIL;
  }
}
      
                         
    
    