import junit.framework.TestCase;

/** A test class for the BiList collection classes. */
public class TreeMapTest extends TestCase {
  
  static final TreeMap<Integer, String> EMPTY = new TreeMap<Integer, String>();
  static final TreeMap<Integer, String> TWO = new TreeMap<Integer, String>();
  static final TreeMap<Integer, String> TWO_FIVE = new TreeMap<Integer, String>();
  public void setUp() {
    TWO.clear();
    TWO.put(2, "two"); 
    TWO_FIVE.clear();
    TWO_FIVE.put(2, "two");
    TWO_FIVE.put(5, "five");
  }
  
  /** Tests the Empty state of TreeMap. */
  public void testIsEmptyAndClear() {
    assertTrue("Empty isEmpty", EMPTY.isEmpty());
    assertTrue("TWO is non-empty", ! TWO.isEmpty());
    assertTrue("TWO_FIVE is non-empty", ! TWO_FIVE.isEmpty());
    TWO.clear();
    assertTrue("TWO isEmpty", TWO.isEmpty());
    TWO_FIVE.clear();
    assertTrue("TWO_FIVE isEmpty", TWO_FIVE.isEmpty());
  }
  
  public void testToString() {
    assertEquals("Print Empty", "{}", EMPTY.toString());
    assertEquals("Print TWO", "{[2,two]}", TWO.toString());
    assertEquals("Print TWO_FIVE", "{[2,two] [5,five]}", TWO_FIVE.toString());
  }
  
  public void testNewMap() {
    assertTrue("newMap() is empty", TWO.newMap().isEmpty());
    assertEquals("newMap has size 0", 0, TWO.newMap().size());
    assertTrue("newMap() is distinct", EMPTY != EMPTY.newMap());
  }
  
  public void testRemoveAndFirstLast() {
    assertEquals("EMPTY initialized", "{}", EMPTY.toString());
    assertEquals("EMPTY has size 0", 0, EMPTY.size());
    assertEquals("EMPTY remove(0)", null, EMPTY.remove(0));
    assertEquals("TWO initialized", "{[2,two]}", TWO.toString());
    assertEquals("TWO has size 1", 1, TWO.size());
    assertEquals("TWO remove(2)", "two", TWO.remove(2));
    assertTrue("TWO empty", TWO.isEmpty());
    assertEquals("TWO has size 0", 0, TWO.size());
    TWO.put(2, "two");
    assertEquals("TWO retored", "{[2,two]}", TWO.toString());
    assertEquals("TWO_FIVE has size 2", 2, TWO_FIVE.size());
    assertEquals("TWO has size 1", 1, TWO.size());
    assertEquals("TWO_FIVE remove(2)", "two", TWO_FIVE.remove(2));
    assertEquals("TWO_FIVE", "{[5,five]}", TWO_FIVE.toString());
    assertEquals("TWO_FIVE has size 1", 1, TWO_FIVE.size());
    TWO_FIVE.put(2, "two");
    assertEquals("TWO_FIVE restored", "{[2,two] [5,five]}", TWO_FIVE.toString());
    assertEquals("TWO_FIVE has size 2", 2, TWO_FIVE.size());
    assertEquals("TWO remove(2)", "two", TWO.remove(2));
    assertTrue("TWO empty", TWO.isEmpty());
    assertEquals("TWO_FIVE remove(5)", "five", TWO_FIVE.remove(5));
    assertEquals("TWO_FIVE diminished by 5", "{[2,two]}", TWO_FIVE.toString());
    
    assertEquals("TWO_FIVE diminished by 5", "{[2,two]}", TWO_FIVE.toString());
    TWO_FIVE.put(5, "five");
    assertEquals("TWO_FIVE restored 1", "{[2,two] [5,five]}", TWO_FIVE.toString());
    assertEquals("TWO_FIVE remove(2)", "two", TWO_FIVE.remove(2));
    assertEquals("TWO_FIVE\\{2}", "{[5,five]}", TWO_FIVE.toString());
    assertEquals("TWO_FIVE remove(5)", "five", TWO_FIVE.remove(5));
    assertEquals("TWO_FIVE\\{2,5}", "{}", TWO_FIVE.toString()); 
    TWO_FIVE.put(5, "five");
    TWO_FIVE.put(2, "two");
    assertEquals("TWO_FIVE restored 2", "{[2,two] [5,five]}", TWO_FIVE.toString());
    TWO_FIVE.put(10, "ten");
    assertEquals("TWO_FIVE augmented by 10", "{[2,two] [5,five] [10,ten]}", TWO_FIVE.toString());
    assertEquals("TWO_FIVE + 10.remove(5)", "five", TWO_FIVE.remove(5));
    assertEquals("TWO_FIVE + 10 diminished by 5", "{[2,two] [10,ten]}", TWO_FIVE.toString());
    
    TreeMap<Integer, String> ZERO_TO_10 = new TreeMap<Integer, String>();
    ZERO_TO_10.put(0, "nought");
    ZERO_TO_10.put(5, "five");
    ZERO_TO_10.put(8, "eight");
    ZERO_TO_10.put(1, "one");
    ZERO_TO_10.put(9, "nine");
    ZERO_TO_10.put(4, "four");
    ZERO_TO_10.put(7, "seven");
    ZERO_TO_10.put(2, "two");
    ZERO_TO_10.put(10, "ten");
    ZERO_TO_10.put(3, "three");
    ZERO_TO_10.put(6, "six");
    assertEquals("ZERO_TO_10 toString()", "{[0,nought] [1,one] [2,two] [3,three] [4,four] [5,five] " +
                 "[6,six] [7,seven] [8,eight] [9,nine] [10,ten]}", ZERO_TO_10.toString());
    assertEquals("ZERO_TO_10.size()", 11, ZERO_TO_10.size());
//    assertEquals("ZERO_TO_10.firstKey()", 0, (int) ZERO_TO_10.firstKey());
//    assertEquals("ZERO_TO_10.lastKey()", 10, (int) ZERO_TO_10.lastKey());
    assertEquals("ZERO_TO_10.remove(10)", "ten", ZERO_TO_10.remove(10));
    assertEquals("ZERO_TO_10\\{10}.toString()", "{[0,nought] [1,one] [2,two] [3,three] [4,four] [5,five] " +
                 "[6,six] [7,seven] [8,eight] [9,nine]}", ZERO_TO_10.toString());
//    assertEquals("ZERO_TO_10\\{10}.lastKey", 9, (int) ZERO_TO_10.lastKey());
    assertEquals("ZERO_TO_10 size 2", 10, ZERO_TO_10.size());
    ZERO_TO_10.put(10,"ten");
    assertEquals("ZERO_TO_10.size() #2", 11, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10 restored", "{[0,nought] [1,one] [2,two] [3,three] [4,four] [5,five] " +
                 "[6,six] [7,seven] [8,eight] [9,nine] [10,ten]}", ZERO_TO_10.toString());
    ZERO_TO_10.put(0,"zero");
    assertEquals("ZERO_TO_10.size() #3", 11, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10 patched", "{[0,zero] [1,one] [2,two] [3,three] [4,four] [5,five] " +
                 "[6,six] [7,seven] [8,eight] [9,nine] [10,ten]}", ZERO_TO_10.toString());
    assertEquals("ZERO_TO_10.remove(5)", "five", ZERO_TO_10.remove(5));
    assertEquals("ZERO_TO_10\\{5}.size()", 10, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{5}.toString()", "{[0,zero] [1,one] [2,two] [3,three] [4,four] " +
                 "[6,six] [7,seven] [8,eight] [9,nine] [10,ten]}", ZERO_TO_10.toString());
    assertEquals("ZERO_TO_10\\{5}.remove(0)", "zero", ZERO_TO_10.remove(0));
    assertEquals("ZERO_TO_10.size() #4", 9, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,5}.toString()", "{[1,one] [2,two] [3,three] [4,four] " +
                 "[6,six] [7,seven] [8,eight] [9,nine] [10,ten]}", ZERO_TO_10.toString());
//    assertEquals("ZERO_TO_10\\{0,5}.firstKey()", 1, (int) ZERO_TO_10.firstKey());
    assertEquals("ZERO_TO_10\\{0,5}.remove(8)", "eight", ZERO_TO_10.remove(8));
    assertEquals("ZERO_TO_10.size() #5", 8, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,5,8}.remove(4)", "four", ZERO_TO_10.remove(4));
    assertEquals("ZERO_TO_10\\{0,4,5,8}.size()", 7, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,4,5,8}.toString()", "{[1,one] [2,two] [3,three] " +
                 "[6,six] [7,seven] [9,nine] [10,ten]}", ZERO_TO_10.toString());
    assertEquals("ZERO_TO_10\\{0,4,5,8}.remove(10)", "ten", ZERO_TO_10.remove(10));
    assertEquals("ZERO_TO_10\\{0,4,5,8,10}.size()", 6, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,4,5,8,10}.toString()", "{[1,one] [2,two] [3,three] " +
                 "[6,six] [7,seven] [9,nine]}", ZERO_TO_10.toString());
    assertEquals("ZERO_TO_10\\{0,4,5,8,10}.remove(3)", "three", ZERO_TO_10.remove(3));
    assertEquals("ZERO_TO_10\\{0,3,4,5,8,10}.size()", 5, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,3,4,5,8,10}.remove(9)", "nine", ZERO_TO_10.remove(9));
    assertEquals("ZERO_TO_10\\{0,3,4,5,8,9,10}.size()", 4, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,3,4,5,8,9,10}.toString()", "{[1,one] [2,two] [6,six] [7,seven]}", 
                 ZERO_TO_10.toString());
    assertEquals("ZERO_TO_10\\{0,3,4,5,8,9,10}.remove(6)", "six", ZERO_TO_10.remove(6));
    assertEquals("ZERO_TO_10\\{0,3,4,5,6,8,9,10}.size()", 3, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,3,4,5,6,8,9,10}.remove(7)", "seven", ZERO_TO_10.remove(7));
    assertEquals("ZERO_TO_10\\{0,3,4,5,6,7,8,9,10}.size()", 2, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,3,4,5,6,7,8,9,10}.toString()", "{[1,one] [2,two]}", ZERO_TO_10.toString());
    assertEquals("ZERO_TO_10\\{0,3,4,5,6,7,8,9,10}.remove(2)", "two", ZERO_TO_10.remove(2));
    assertEquals("ZERO_TO_10\\{0,2,3,4,5,6,7,8,9,10}.size()", 1, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,2,3,4,5,6,7,8,9,10}.remove(1)", "one", ZERO_TO_10.remove(1));
    assertEquals("ZERO_TO_10\\{0,1,2,3,4,5,6,7,8,9,10}.size()", 0, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{0,1,2,3,4,5,6,7,8,9,10}.toString()", "{}", ZERO_TO_10.toString());
    assertEquals("ZERO_TO_10\\{0,1,2,3,4,5,6,7,8,9,10}.remove(0)", null, ZERO_TO_10.remove(0));
  }
  
  /** Tests get and put. */
  public void testGetPutSizeFirstLast() {
    assertEquals("EMPTY get(0)", null, EMPTY.get(0));
    assertEquals("EMPTY.size()", 0, EMPTY.size());
    assertEquals("TWO get(2)", "two", TWO.get(2));
    assertEquals("TWO get(1)", null, TWO.get(1));
    assertEquals("TWO_FIVE.get(2)", "two", TWO_FIVE.get(2));
    assertEquals("TWO_FIVE.get(5)", "five", TWO_FIVE.get(5));
//    assertEquals("TWO_FIVE.firstKey()", 2, (int) TWO_FIVE.firstKey());
//    assertEquals("TWO_FIVE.lastKey()", 5, (int) TWO_FIVE.lastKey());
    assertEquals("TWO.size()", 1, TWO.size());
    assertEquals("TWO_FIVE.size()", 2, TWO_FIVE.size());
    assertEquals("TWO_FIVE.put(3)", null, TWO_FIVE.put(3, "three"));
    assertEquals("TWO_FIVE get(3)", "three", TWO_FIVE.get(3));
    assertEquals("TWO_FIVE.size() #2", 3, TWO_FIVE.size());
    assertEquals("TWO_THREE_FIVE toString", "{[2,two] [3,three] [5,five]}", TWO_FIVE.toString());
    assertEquals("TWO_FIVE.put(3)", "three", TWO_FIVE.put(3,"tres"));
    assertEquals("TWO_FIVE get(3) #2", "tres", TWO_FIVE.get(3));                
  }
}
