import junit.framework.TestCase;

/** The test class for MyHashMap. */
public class MyHashMapTest extends TestCase {
  
  /** Test method for get, put, size. */
  MyHashMap<String, Integer> EMPTY;
  MyHashMap<String, Integer> TWO;
  MyHashMap<String, Integer> TWO_FIVE;
  
  public void setUp() {
    EMPTY = new MyHashMap<String, Integer>();
    TWO = new MyHashMap<String, Integer>();
    TWO.put("two", 2);
    TWO_FIVE = new MyHashMap<String, Integer>();
    TWO_FIVE.put("two", 2);
    TWO_FIVE.put("five", 5);
  }
  
  public void testIsEmptyAndClear() {
    assertTrue("Empty isEmpty", EMPTY.isEmpty());
    assertTrue("TWO is non-empty", ! TWO.isEmpty());
    assertTrue("TWO_FIVE is non-empty", ! TWO_FIVE.isEmpty());
    TWO.clear();
    assertTrue("TWO isEmpty", TWO.isEmpty());
    TWO_FIVE.clear();
    assertTrue("TWO_FIVE isEmpty", TWO_FIVE.isEmpty());
  }
  
   public void testNewMap() {
    assertTrue("newMap() is empty", TWO.newMap().isEmpty());
    assertEquals("newMap has size 0", 0, TWO.newMap().size());
    assertTrue("newMap() is distinct", EMPTY != EMPTY.newMap());
  }
   
   public void testRemoveAndFirstLast() {
     assertEquals("EMPTY has size 0", 0, EMPTY.size());
     assertEquals("EMPTY remove(zero)", null, EMPTY.remove("zero"));
     assertEquals("TWO has size 1", 1, TWO.size());
     assertEquals("TWO remove(two)", (Integer) 2, TWO.remove("two"));
     assertTrue("TWO empty", TWO.isEmpty());
     assertEquals("TWO has size 0", 0, TWO.size());
     TWO.put("two", 2);
     assertEquals("TWO has size 1", 1, TWO.size());
     assertEquals("TWO_FIVE remove(two)", (Integer) 2, TWO_FIVE.remove("two"));
     assertEquals("TWO_FIVE has size 1", 1, TWO_FIVE.size());
     TWO_FIVE.put("two", 2);
     assertEquals("TWO_FIVE has size 2", 2, TWO_FIVE.size());
     assertEquals("TWO remove(two)", (Integer) 2, TWO.remove("two"));
     assertTrue("TWO empty", TWO.isEmpty());
     assertEquals("TWO_FIVE remove(five)", (Integer) 5, TWO_FIVE.remove("five"));
     assertEquals("TWO_FIVE has size 1", 1, TWO_FIVE.size());
     assertEquals("TWO_FIVE contains two", (Integer) 2, TWO_FIVE.get("two"));
     TWO_FIVE.put("five", 5);
     assertEquals("TWO_FIVE contains five", (Integer) 5, TWO_FIVE.get("five"));
     assertEquals("TWO_FIVE remove(two)", (Integer) 2, TWO_FIVE.remove("two"));
     assertEquals("TWO_FIVE\\{2}", 1, TWO_FIVE.size());
     assertEquals("TWO_FIVE contains five", 5, (int) TWO_FIVE.get("five"));
     assertEquals("TWO_FIVE does not contain two", null, TWO_FIVE.get("two"));
     assertEquals("TWO_FIVE remove(five)", 5, (int) TWO_FIVE.remove("five"));
     assertEquals("TWO_FIVE\\{2,5}", 0, TWO_FIVE.size());
     assertEquals("TWO_FIVE does not contain five", null, TWO_FIVE.get("five"));
     assertEquals("TWO_FIVE does not contain two", null, TWO_FIVE.get("two"));
     TWO_FIVE.put("five", 5);
     TWO_FIVE.put("two", 2);
     assertEquals("TWO_FIVE restored 2", 2, (int) TWO_FIVE.get("two"));
     assertEquals("TWO_FIVE restored 5", 5, (int) TWO_FIVE.get("five"));
     TWO_FIVE.put("ten", 10);
     assertEquals("TWO_FIVE augmented by 10", 3, TWO_FIVE.size());
     assertEquals("TWO_FIVE contains 10", 10, (int) TWO_FIVE.get("ten"));
     assertEquals("TWO_FIVE + 10.remove(five)", 5, (int) TWO_FIVE.remove("five"));
     assertEquals("TWO_FIVE + 10 diminished by 5", 2, TWO_FIVE.size());
     assertEquals("TWO_FIVE contains 10", 10, (int) TWO_FIVE.get("ten"));
     assertEquals("TWO_FIVE contains 2", 2, (int) TWO_FIVE.get("two"));
     assertEquals("TWO_FIVE does not contain 5", null, TWO_FIVE.get("five"));
     
    MyHashMap<Integer, String> ZERO_TO_10 = new MyHashMap<Integer, String>();
    ZERO_TO_10.put(0, "nought");
    ZERO_TO_10.put(5, "five");
    ZERO_TO_10.put(8, "eight");
    ZERO_TO_10.put(1, "one");
    ZERO_TO_10.put(9, "nine");
    ZERO_TO_10.put(4, "four");
    ZERO_TO_10.put(7, "seven");
    ZERO_TO_10.put(2, "two");
    ZERO_TO_10.put(10, "ten");
    ZERO_TO_10.put(3, "three");
    ZERO_TO_10.put(6, "six");
    assertEquals("ZERO_TO_10 size()", 11, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10.get(0)", "nought", ZERO_TO_10.get(0));
    assertEquals("ZERO_TO_10.get(7)", "seven", ZERO_TO_10.get(7));
    assertEquals("ZERO_TO_10.remove(10)", "ten", ZERO_TO_10.remove(10));
    assertEquals("ZERO_TO_10 size is 10", 10, ZERO_TO_10.size());
    ZERO_TO_10.put(10, "ten");
    assertEquals("ZERO_TO_10.size() is 11", 11, ZERO_TO_10.size());

    ZERO_TO_10.put(0,"zero");
    assertEquals("ZERO_TO_10.size() #3", 11, ZERO_TO_10.size());

    assertEquals("ZERO_TO_10.remove(5)", "five", ZERO_TO_10.remove(5));
    assertEquals("ZERO_TO_10\\{5}.size()", 10, ZERO_TO_10.size());
    assertEquals("ZERO_TO_10\\{5}.remove(0)", "zero", ZERO_TO_10.remove(0));
    assertEquals("ZERO_TO_10\\{0,5}.size()", 9, ZERO_TO_10.size());
  }
   
 /** Tests get and put. */
  public void testGetPutSizeContains() {
    assertEquals("EMPTY get(zero)", null, EMPTY.get("zero"));
    assertEquals("EMPTY.contains(zero)", false, EMPTY.containsKey("zero"));
    assertEquals("EMPTY.size()", 0, EMPTY.size());
    assertEquals("TWO get(two)", 2, (int) TWO.get("two"));
    assertEquals("TWO get(one)", null, TWO.get("one"));
    assertEquals("TWO.contains(two)", true, TWO.containsKey("two"));
    assertEquals("TWO.contains(one)", false, TWO.containsKey("one"));
    assertEquals("TWO_FIVE.get(two)", 2, (int) TWO_FIVE.get("two"));
    assertEquals("TWO_FIVE.get(five)", 5, (int) TWO_FIVE.get("five"));
    assertEquals("TWO.contains(five)", true, TWO_FIVE.containsKey("five"));
    assertEquals("TWO.size()", 1, TWO.size());
    assertEquals("TWO_FIVE.size()", 2, TWO_FIVE.size());
    assertEquals("TWO_FIVE.put(three)", null, TWO_FIVE.put("three", 3));
    assertEquals("TWO_FIVE get(three)", 3, (int) TWO_FIVE.get("three"));
    assertEquals("TWO_FIVE.contains(three)", true, TWO_FIVE.containsKey("three"));
    assertEquals("TWO_FIVE.contains(four)", false, TWO_FIVE.containsKey("four"));
    assertEquals("TWO_FIVE.size() #2", 3, TWO_FIVE.size());
    assertEquals("TWO_FIVE.put(funnyThree)", 3, (int) TWO_FIVE.put("three", 30));
    assertEquals("TWO_FIVE get(three) #2", 30, (int) TWO_FIVE.get("three"));                
  }
}
