import junit.framework.TestCase;

/** A test class for the BiList collection classes. */
public class BigBiListTest extends TestCase {
  
  static BiList<Integer> EMPTY;
  static BiList<Integer> TWO;
  static BiList<Integer> TWO_FIVE;
  
  public void setUp() {
    EMPTY = new BiList<Integer>();
    TWO = new BiList<Integer>().insertFront(2);
    TWO_FIVE = new BiList<Integer>().insertFront(2).insertRear(5);
  }
  
  /** Tests the Empty state of BiList. */
  public void testIsEmpty() {
    assertTrue("Empty isEmpty", EMPTY.isEmpty());
    assertTrue("TWO is non-empty", ! TWO.isEmpty());
    assertTrue("TWO_FIVE is non-empty", ! TWO_FIVE.isEmpty());
  }
  
  public void testToString() {
    assertEquals("Print Empty", "()", EMPTY.toString());
    assertEquals("Print TWO", "(2)", TWO.toString());
  }
  
  public void testListInsertRemAndLength() { 
    BiListI<Integer> b = new BiList<Integer>();
    assertEquals("b is empty", "()", b.toString());
    assertEquals("b has zero length", 0, b.length());
    assertEquals("b is '(2)'", "(2)", b.insertFront(2).toString());
    assertEquals("b has length 1", 1, b.length());
    assertEquals("b is '(1 2)'", "(1 2)", b.insertFront(1).toString());
    assertEquals("b has length 2", 2, b.length());
    assertEquals("b is '(1 2 3)'", "(1 2 3)", b.insertRear(3).toString());
    assertEquals("b has length 3", 3, b.length());
    int last = b.remRear();
    assertEquals("last is 3", 3, last);
    assertEquals("b is '(1 2)'", "(1 2)", b.toString());
    assertEquals("b has length 2", 2, b.length());
    last = b.remRear();
    assertEquals("last is 2", 2, last);
    assertEquals("b is '(1)'", "(1)", b.toString());
    assertEquals("b has length 1", 1, b.length());
    last = b.remRear();
    assertEquals("last is 1", 1, last);
    assertEquals("b is empty", "()", b.toString());
    assertEquals("b has length 0", 0, b.length());

    assertEquals("b is '(2)'", "(2)", b.insertRear(2).toString());
    assertEquals("b is '(2 3)'", "(2 3)", b.insertRear(3).toString());
    assertEquals("b is '(1 2 3)'", "(1 2 3)", b.insertFront(1).toString());
    int first = b.remFront();
    assertEquals("first is 1", 1, first);
    assertEquals("b is '(2 3)'", "(2 3)", b.toString());
    first = b.remFront();
    assertEquals("first is 2", 2, first);
    assertEquals("b is '(3)'", "(3)", b.toString());
    first = b.remFront(); 
    assertEquals("first is 3", 3, first);
    assertEquals("b is empty", "()", b.toString());
  }
  
  public void testNewListAndIterator() {
    assertTrue("newList() is empty", TWO.newList().isEmpty());
    assertTrue("newList() is distinct", ! (EMPTY == EMPTY.newList()));
    BiIteratorI<Integer> it = TWO_FIVE.newIterator();
    int current = it.currentItem();
    assertEquals("current = 2", 2, current);
    assertTrue("it atStart()", it.atStart());
    assertTrue("it not atEnd()", ! it.atEnd());
    assertEquals("next is 5", 5, (int) it.next());
    current = it.currentItem();
    assertEquals("current = 5", 5, current);
    assertTrue("it not atStart()", ! it.atStart());
    assertTrue("it not atEnd()", ! it.atEnd());
    it.next();
    assertTrue("it atEnd()", it.atEnd());
    it.insert(10);
    assertEquals("TWO_FIVE = '(2 5 10)'", "(2 5 10)", TWO_FIVE.toString());
    assertTrue("it atEnd()", it.atEnd());
    assertEquals("length is 3", 3, TWO_FIVE.length());
    it.first();
    assertTrue("it atStart()", it.atStart());
    current = it.currentItem();
    assertEquals("current = 2", 2, current);
    it.last();
    current = it.currentItem();
    assertEquals("current = 10", 10, current);
    it.remove();
    assertTrue(it.atEnd());
    assertEquals("length = 2", 2, TWO_FIVE.length());
    it.prev();
    current = it.currentItem();
    assertEquals("current = 5", 5, current);
    it.remove();
    assertTrue("it atEnd", it.atEnd());
    it.prev();
    assertTrue("it atStart", it.atStart());
    it.remove();
    assertTrue("it not atStart", ! it.atStart());
    assertTrue("it atEnd (list is circular)", it.atEnd());
    try { 
      it.remove();
      fail("Exception not thrown for remove with no currentItem");
    }
    catch(IteratorException e) { /* silently succeed */ }
    try { 
      it.currentItem();
      fail("Exception not thrown for currentItem with no currentItem");
    }
    catch(IteratorException e) { /* silently succeed */ }
  }
  
  public void testMap() {
    ILambda<Integer, Integer> square = new ILambda<Integer, Integer>() {
      public Integer apply(Integer host) { return host.intValue()*host.intValue(); }
    };
    BiList<Integer> bl = TWO_FIVE.map(square);
    assertEquals("map x -> x*x to TWO_FIVE", "(4 25)", bl.toString());
  }
  /** Note this class does not test wraparound for next() and prev(), which BiList implements. */

}
