// Language Level Converter line number map: dj*->java. Entries: 37
//     1->7         2->8         3->9         4->10        5->11        6->12        7->13        8->14   
//     9->15       10->16       11->17       12->48       13->49       14->50       15->51       16->52   
//    17->53       18->54       19->55       20->56       21->57       22->58       23->84       24->85   
//    25->86       26->87       27->88       28->89       29->90       30->91       31->92       32->93   
//    33->94       34->95       35->96       36->97       37->145  
/** Composite representation of IntList ::= EmptyIntList | ConsIntList(int, IntList) with visitor interface. 
  * The interface is defined in a different file. */
abstract class IntList {
  public abstract Object accept(final IntListVisitor v);
  /** Sorts this IntList into ascending (non-descending) order. */
  public abstract IntList sort();
  /** Adds the int n to the front of this IntList. */
  public IntList cons(final int n) { return new ConsIntList(n, this); }
  /** Inserts n in proper order in this IntList, assuming this is sorted in ascending order. */
  public abstract IntList insert(final int n);

  /** This method is automatically generated by the Language Level Converter. */
  public IntList() {
    super();
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      IntList cast = ((IntList) o);
      return true;
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode();
  }
}

class EmptyIntList extends IntList { 
  /** Field containing the only (singleton) EmptyIntList. */
  public final static EmptyIntList ONLY = new EmptyIntList();
  /** Constructor used to create ONLY. */
  private EmptyIntList() { }

  public Object accept(final IntListVisitor v) { return v.forEmptyIntList(this); }
  public IntList sort() { return this; }
  public IntList insert(final int n) { return cons(n); }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      EmptyIntList cast = ((EmptyIntList) o);
      return true;
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode();
  }
}

class ConsIntList extends IntList {
  /** The first element of this. */
  private final int first;
  /** The remaining elements of this. */
  private final IntList rest;
  
  public Object accept(final IntListVisitor v) { return v.forConsIntList(this); }
  public IntList sort() { return rest.sort().insert(first); }
  public IntList insert(final int n) {
    if (n <= first) return cons(n);
    else return rest.insert(n).cons(first);
  }

  /** This method is automatically generated by the Language Level Converter. */
  public ConsIntList(int first, IntList rest) {
    super();
    this.first = first;
    this.rest = rest;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int first() {
    return first;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public IntList rest() {
    return rest;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        first() + ", " + 
        rest() + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      ConsIntList cast = ((ConsIntList) o);
      return (first() == cast.first()) && 
          (rest() != null && rest().equals(cast.rest()));
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode() ^ 
        first() ^ 
        (rest() == null ? 0 : rest().hashCode());
  }
}

